/* dcf_control.c
 from dc fighting v0.2.2
  by
   jimbo
    Dan Potter and Jordan DeLong of Cryptic Allusion
     Andrew K.
      Marcus Comstedt


 dcf_globals.c : the root node of the dcfighting source tree.
 |
 | dependencies:
  - dcf_animation.c
  - dcf_consequence.c
  - dcf_control.c
  - dcf_draw.c
  - dcf_fighter.c
  - dcf_mainloop.c
  - dcf_player.c

   something to remember: these libs use negative radians in glRotatef()
    relative to my linux/NVidia GL libs.


   using
   KallistiOS 1.1.5

   adapted from
   gltest.c
   (c)2001 Dan Potter
*/

#include <kos.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include <stdlib.h>

/*** for interpolation : a move queue ***/
#define HISTORY_SIZE 16
#define FUTURE_SIZE 16

/* these are extra padded variables at the beginning of each anchor
 *  - gets me out of having to define a struct anchor
 *
 */ 
#define MOVE_AGO( x )  p->change_history[ (p->change_pointer - x + HISTORY_SIZE) % HISTORY_SIZE ]
#define TIME_OF( x )  p->future_lists[0][ ( p->future_pointer + x + FUTURE_SIZE) % FUTURE_SIZE ]
#define STATE_OF( x ) p->future_lists[1][ ( p->future_pointer + x + FUTURE_SIZE) % FUTURE_SIZE ]
/*** v0.1.9 - for partials ***/
#define PARAMS_OF( x ) p->future_lists[2][ ( p->future_pointer + x + FUTURE_SIZE) % FUTURE_SIZE ]
#define OFFSET_HACK 4

// 32 for the new fighter model(v0.2.1)...  0 for old(v0.0.0-v0.2.0)
#define SPACIAL_OFFSET  32

/*** v0.2.3 ***/
#define RUNSTATE_FIGHT  0
#define RUNSTATE_PAUSE  1
#define RUNSTATE_MENU   2
#define RUNSTATE_VICTORY   3

/* here are the global variables
 *
 *
 *
 */
extern uint8 romdisk[];

/*** for view ***/
double view_rot = 0.0;
double view_roty = 0.0;
/*** for manipulation ***/
int chosen_param = 0;
/*** for interpolation ***/
double t;
double timestep = 0.02;

/*** camera/display parameters ***/
float look_at[6] = { 0.0, 12.0, -10.0, 0.0, 1.0, 0.0 };
int camera_follow = 1;
int display_frame_rate = 1;
int block_damage = 1;
int use_stamina = 1;

int menu_selection = 0;

/*** v0.1.8 : slow-mo for victory ***/
int slomo_counter = 0;





/* v0.2.2: numbers for tweaking
 *
 *   hope to contain all dcfg state variables here
 *    maybe even in array form, with an array of names.
 *     that way a quick and small menu could change every parameter.
 *
 */
double throw_distance = 5.5;
double short_distance = 4.5;
double medium_distance = 5.5;
double long_distance = 7.0;
double default_distance = 10.0;
/*** scale of up/down/left/right motion in world ***/
double sidestep_length = 0.08;
double towardstep_length = 0.4;

/*** stamina tweak ***/
int minimum_stamina = 30;
double stamina_coeffiecient = (5.0/300.0);



/* v0.2.2: angle tolerance for attacks 
 *   in radians
 *    0.5rad ~= 30deg
 */
double attack_direction_tolerance = 0.5;
double block_direction_tolerance = 1.5;






/*** v0.2.3: for menuss, etc ***/
int running_state = RUNSTATE_FIGHT;


/*** v0.2.4 : moved out here ***/
int controller_count = 0;
int player_count = 0;



/* vector dot and cross product in 3D only
 *
 */
inline double dot_product( double *a, double *b ) {
  return a[0] * b[0]  + a[1] * b[1]  + a[2] * b[2];
}
inline void cross_product( double *a, double *b, double *c ) {


  c[0]  =  b[1] * a[2]  -  a[1] * b[2];
  c[1]  =  b[2] * a[0]  -  b[0] * a[2];
  c[2]  =  b[0] * a[1]  -  a[0] * b[1];


#if 0

  c[0]  =  a[1] * b[2] -  a[2] * b[1];
  c[1]  =  a[0] * b[2] -  a[2] * b[0];
  c[2]  =  a[0] * b[1] -  a[1] * b[0];


#endif
}



/*** for text display ***/
#include "dcf_draw.c"
#include "cube.c"
#include "dcf_fighter.c"
//#include "dcf_fighter2.c"
/*** refer to some(all?) of the above globals ***/
#include "dcf_player.c"
#include "dcf_control.c"
#include "dcf_animation.c"
#include "dcf_consequence.c"
#include "dcf_mainloop.c"



/* v0.2.4: reset fighting conditions
 *    assumes 2 players for now
 *
 */
void reset_fight() {


  reset_fighter( player1 );
  reset_fighter( player2 );
  
  // position reset
  player1->position[0] = -5.0;
  player2->position[0] = 5.0;
  player1->position[2] = 0.0;
  player2->position[2] = 0.0;
  
  
  
  
  line_up_opponent( player1 );
  line_up_opponent( player2 );
  

}


