/* dcf_control.c
 from dc fighting v0.2.0
  by
   jimbo
    Dan Potter and Jordan DeLong of Cryptic Allusion
     Andrew K.
      Marcus Comstedt


 contains the main() function for dcfighting:
 ----------------------------

 init ( kos, matrix stacks )
 load textures
 init fighters
 fill reverse of arrays
 find controllers
 while(1)
 {
 
   read controller
   if state == fighting     control function
   if state == fighting || victory    animation timestep
   if state == fighting || victory     consequence
   draw scene 
   (
     background
     bars
     ground
     fighters
   );
 }




*/





int main(int argc, char **argv) {
  int i;
  int players[4][2];

  /*** need this for #define macros ***/
  fighter_t *p;

  /*** controller variables ***/
  cont_cond_t cond, cond2, cond3, cond4;
  uint8	c, c2, c3, c4;


  /*** texture pointers ***/
  GLuint texture;
  GLuint texture2;
  GLuint texture3;
  GLuint texture4;


  /*** v0.2.2: frame counter ***/
  int frame_count = 0;

  int paused = 0;
  int transpause = 0;



  time_t begin_time;
  time_t last_time;
  time_t time_value;
  double frames_per_second;
  
  char screenmes[256];


/*** Dan Potter's code ***/
  /* Initialize KOS */
  kos_init_all( THD_ENABLE | IRQ_ENABLE | TA_ENABLE, romdisk );
  /* Get basic stuff initialized */
  glKosInit();
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(30.0f, 640.0f / 480.0f, 0.01f, 1000.0f);
  glMatrixMode(GL_MODELVIEW);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_KOS_AUTO_UV);
  
  /* Load a texture */
  //loadtxr("/rd/crate.pcx", &texture);
  //  loadtxr("/rd/brain.pcx", &texture);
  // loadtxr("/rd/roots.pcx", &texture);
  // loadtxr("/rd/bluedragon.pcx", &texture);
  // loadtxr("/rd/splash.pcx", &texture);
  //loadtxr("/rd/barbecue.pcx", &texture);
  //loadtxr("/rd/spruce.pcx", &texture);
  // loadtxr("/rd/skull.pcx", &texture);
  loadtxr("/rd/brain.pcx", &texture);
  loadtxr("/rd/sunrise.pcx", &texture2);
  loadtxr("/rd/crate.pcx", &texture3);
  loadtxr("/rd/crate.pcx", &texture4);

  printf("texture  is %08x\n", texture);
  printf("texture2 is %08x\n", texture2);
  printf("texture3 is %08x\n", texture3);
  printf("texture4 is %08x\n", texture4);

  printf("\n\nstarting init...\n\n");
/*** jimbo ***/


  /*** fighter init ***/
  player1 = malloc( sizeof( fighter_t ) );
  player2 = malloc( sizeof( fighter_t ) );
  player3 = malloc( sizeof( fighter_t ) );
  player4 = malloc( sizeof( fighter_t ) );


  init_fighter( player1 );
  init_fighter( player2 );
  init_fighter( player3 );
  init_fighter( player4 );

  player1->opponent = player2;
  player2->opponent = player1;
  player3->opponent = player4;
  player4->opponent = player3;


#if 0

  // fighter  specific init
  player1->position[0] = -5.0;
  player2->position[0] = 5.0;
  player3->position[2] = -5.0;
  player4->position[2] = 5.0;



  /*** v0.2.0 ***/
  line_up_opponent( player1 );
  line_up_opponent( player2 );
  line_up_opponent( player3 );
  line_up_opponent( player4 );
#endif


  reset_fight();


  /* where's the second controller?
   * input device check - scan bus
   *
   *
   */
  for ( i=0; i<4; i++ ) {
    int j;
    for ( j=0; j<4; j++ ) {
      int k; for ( k=0; k<j; k++ ) { printf(" "); }
      printf("Device %d %d)   %d\n", i, j, maple_device_func(i,j) );
      /*** found controller (thanks Dan and Marcus) ***/
      if ( maple_device_func(i,j) == MAPLE_FUNC_CONTROLLER ) {
	/*** 1st player ***/
	if ( player_count == 0 ) {
	  players[0][0] = i;
	  players[0][1] = j;
	  player_count++;
	}
	/*** just 2p for now ***/
	else {
	  players[1][0] = i;
	  players[1][1] = j;
	  player_count++;
	}
      }
    }
  }
  /*** v0.2.4 ***/
  controller_count = player_count;

  c = maple_create_addr( players[0][0] , players[0][1] );
  printf("c == %d\n", c );

  if ( player_count >= 2 ) {
    c2 = maple_create_addr( players[1][0] , players[1][1] ); 
    printf("c2 == %d\n", c2 );
  }
  if ( player_count >= 4 ) {
    c3 = maple_create_addr( players[2][0] , players[2][1] ); 
    printf("c3 == %d\n", c4 );
  }
  if ( player_count >= 4 ) {
    c4 = maple_create_addr( players[3][0] , players[3][1] ); 
    printf("c4 == %d\n", c4 );
  }



  /* v0.2.2: init text services
   *
   *
   */
  draw_init();  
  timer_init();
  begin_time = rtc_unix_secs();
  

  /* MAIN LOOP:
   *  - get controller input (possibly button combos)
   *  - add it into a queue
   *  - check current controller state and history for moves
   *  - move the entity in the proper manner (interpolate)
   *  - draw it
   *
   */
  running_state = RUNSTATE_FIGHT;
  while(1) {
    int crossed = 0;
    
    

    /* v0.2.2: update frame rate counter
     *    - courtesy of Trilinear
     *
     */
    time_value = rtc_unix_secs();
    if ( time_value != last_time ) {
      int tim = (time_value & 0xFFFF) - (begin_time & 0xFFFF);
      last_time = time_value;
      if ( tim > 0 ) {
	frames_per_second = (double)frame_count/(double)tim;
      }
      else frames_per_second = 0.0;
    }



    /*** Dan Potter's code for players 1 and 2 ***/
    if ( player_count >= 1 ) 
      if (cont_get_cond(c, &cond) < 0) {
	printf("Error reading controller 1\n");
	break;
      }
    if ( player_count >= 2 ) 
      if (cont_get_cond(c2, &cond2) < 0) {
	printf("Error reading controller 2\n");
	break;
      }
    if ( player_count >= 3 ) 
      if (cont_get_cond(c3, &cond3) < 0) {
	printf("Error reading controller 3\n");
	break;
      }
    if ( player_count >= 4 ) 
      if (cont_get_cond(c4, &cond4) < 0) {
	printf("Error reading controller 4\n");
	break;
      }

    /*** reboot condition: ABXYSTART ***/
    if (!(cond.buttons & CONT_START))
      if (!(cond.buttons & CONT_A))
	if (!(cond.buttons & CONT_B))
	  if (!(cond.buttons & CONT_X))
	    if (!(cond.buttons & CONT_Y))
	      break;


    /*** reboot condition: ABXYSTART ***/
    if ( player_count >= 2 ) 
      if (!(cond2.buttons & CONT_START))
	if (!(cond2.buttons & CONT_A))
	  if (!(cond2.buttons & CONT_B))
	    if (!(cond2.buttons & CONT_X))
	      if (!(cond2.buttons & CONT_Y))
		break;




    /* v0.2.2: pause/menu condition
     * v0.2.41: fixed for one controller bug.
     *
     *
     if (   (!(cond.buttons & CONT_START)) 
     ||
     (!(cond2.buttons & CONT_START))    ) {
     */
    if (    (!(cond.buttons & CONT_START)) 
	    ||
	    ( 
	      ( player_count >= 2 ) 
	       && 
	      (!(cond2.buttons & CONT_START))   
	    )
       ) {

      /* This weird logic is to avoid bouncing back
	 and forth before the user lets go of the
	 button. */
      if (!(transpause & 0x1000)) {
	
	
	switch ( running_state ) {

	case RUNSTATE_FIGHT: running_state = RUNSTATE_PAUSE; break;
	case RUNSTATE_PAUSE: running_state = RUNSTATE_FIGHT; break;
	
	  /* v0.2.3
	   *
	   * reset after victory 
	   *  energy, position, rotation
	   *
	   */
	case RUNSTATE_VICTORY:
	  reset_fight();
	  running_state = RUNSTATE_FIGHT; 
	  break;
	  
	  /*** if somewhere else, come back to pause ***/
	default: running_state = RUNSTATE_PAUSE; break;
	  
	}



	if (transpause == 0)
	  transpause = 0x1001;
	else
	  transpause = 0x1000;
      }
    } else {
      transpause &= ~0x1000;
    }
    
    
    
    /*** v0.2.2: check controller if fighting ***/
    if ( running_state == RUNSTATE_FIGHT ) {
      
      /*** check (all) controllers and take appropriate action ***/
      if ( player_count >= 1 ) 
	process_input2( player1, cond );
      else random_input( player1 );
      
      if ( player_count >= 2 ) 
	process_input2( player2, cond2 );
      else random_input( player2 );
      
      if ( player_count >= 3 ) 
	process_input2( player3, cond3 );
    
      if ( player_count >= 4 ) 
	process_input2( player4, cond4 );
      
    }
    
    /* fight and victory - run animation and consequences.
     *
     *
     */
    if ( 
	( running_state == RUNSTATE_FIGHT )
	||
	( running_state == RUNSTATE_VICTORY )
	) {
     
      /*** v0.1.7: getting these functions out would speed us up ***/
      animation_timestep( player1 );
      animation_timestep( player2 );
      if ( player_count >= 3 ) 
	animation_timestep( player3 );
      if ( player_count >= 4 ) 
	animation_timestep( player4 );
      
      /*** consequences every frame ***/
      determine_result( player1, player1->opponent );
      determine_result( player2, player2->opponent );
      if ( player_count >= 3 ) 
	determine_result( player3, player3->opponent );
      if ( player_count >= 4 ) 
	determine_result( player4, player4->opponent );
    }


    /*** paused 
     *
     *
     */
    if ( running_state == RUNSTATE_PAUSE ) {
      /*** check (all) controllers and take appropriate action ***/
      if ( player_count >= 1 ) 
	process_menu_input( player1, cond );
      if ( player_count >= 2 ) 
	process_menu_input( player2, cond2 );
      if ( player_count >= 3 ) 
	process_menu_input( player3, cond3 );
      if ( player_count >= 4 ) 
	process_menu_input( player4, cond4 );
    }
    


    /* Animation done.
     *  Begin frame :
     *   display code below
     */
    glKosBeginFrame();
    
    /*** Draw in space ***/
    glLoadIdentity();
    glPushMatrix();

    /*** v0.2.0 : hacky flat background ***/
    glBindTexture(GL_TEXTURE_2D, texture2 );
    glColor3f(1.0f, 1.0f, 1.0f);
    glBegin(GL_QUADS);  
      glVertex3f(-270.0f,   180.0f,   -300.0f);
      glVertex3f( 270.0f,   180.0f,   -300.0f);
      glVertex3f( 270.0f,  -180.0f,   -300.0f);
      glVertex3f(-270.0f,  -180.0f,   -300.0f);
    glEnd();


    glBindTexture(GL_TEXTURE_2D, texture );
    glColor3f(1.0f, 0.0f, 0.0f);
    /*** player 1 energy/stamina bars ***/
    glBegin(GL_QUADS);
      glVertex3f( -4.0, 3.0, -5.0 );
      glVertex3f( -4.0 + 3.9 * ((double)player1->energy/(double)ENERGY_MAX), 3.0, -5.0 );
      glVertex3f( -4.0 + 3.9 * ((double)player1->energy/(double)ENERGY_MAX), 2.5, -5.0 );
      glVertex3f( -4.0, 2.5, -5.0 );
    glEnd();


    glBegin(GL_QUADS);
      glVertex3f( -4.0, 3.0, -5.0 );
      glVertex3f( -4.0 + 3.9 * ((double)player1->stamina/(double)STAMINA_MAX), 2.4, -5.0 );
      glVertex3f( -4.0 + 3.9 * ((double)player1->stamina/(double)STAMINA_MAX), 2.2, -5.0 );
      glVertex3f( -4.0, 2.5, -5.0 );
    glEnd();
    
    
    glColor3f(0.0f, 1.0f, 1.0f);
    /*** player 2 energy/stamina bars ***/
    glPushMatrix();
      glTranslatef( 8.0, 0.0, 0.0 );
      glBegin(GL_QUADS);
        glVertex3f( -4.0 - 3.9 * ((double)player2->energy/(double)ENERGY_MAX), 3.0, -5.0 );
	glVertex3f( -4.0, 3.0, -5.0 );
	glVertex3f( -4.0, 2.5, -5.0 );
	glVertex3f( -4.0 - 3.9 * ((double)player2->energy/(double)ENERGY_MAX), 2.5, -5.0 );
      glEnd();
    glPopMatrix();

    
    glPushMatrix();
      glTranslatef( 8.0, 0.0, 0.0 );
      glBegin(GL_QUADS);
        glVertex3f( -4.0 - 3.9 * ((double)player2->stamina/(double)STAMINA_MAX), 2.4, -5.0 );
	glVertex3f( -4.0, 3.0, -5.0 );
	glVertex3f( -4.0, 2.5, -5.0 );
	glVertex3f( -4.0 - 3.9 * ((double)player2->stamina/(double)STAMINA_MAX), 2.2, -5.0 );
      glEnd();
    glPopMatrix();


    /* v0.1.9 : camera positioning 
     *   find centroid
     *
     */
    if ( camera_follow >= 1 ) {
      double midpoint[4];
      double vector_p1p2[4];
      double up[] = { 0.0, 1.0, 0.0, 1.0 };
      double camera_vector[4];
      


      midpoint[0] = ( player1->position[0] + player2->position[0] ) / 2.0;
      midpoint[1] = ( player1->position[1] + player2->position[1] ) / 2.0;
      midpoint[2] = ( player1->position[2] + player2->position[2] ) / 2.0;
      vector_p1p2[0] = player1->position[0] - player2->position[0];
      vector_p1p2[1] = player1->position[1] - player2->position[1];
      vector_p1p2[2] = player1->position[2] - player2->position[2];

      cross_product( vector_p1p2, up, camera_vector );


      look_at[0] = 2.0 * camera_vector[0];
      look_at[1] = 3.0;
      look_at[2] = 2.0 * camera_vector[2];
      look_at[3] = midpoint[0];
      look_at[4] = midpoint[1];
      look_at[5] = midpoint[2];

      /* v0.2.4: new camera mode
       *
       *
       */
      if ( camera_follow == 2 ) {
	look_at[0] = 10.0;
	look_at[1] = 5.0;
	look_at[2] = 10.0;
      }
      /* v0.2.4: new camera mode
       *
       *
       */
      if ( camera_follow == 3 ) {
	look_at[0] = 7.0;
	look_at[1] = 7.0;
	look_at[2] = 7.0;
      }

    }



    /*** world viewing transformation ***/
    glewLookAt( look_at[0],
		look_at[1],
		look_at[2],
		look_at[3],
		look_at[4],
		look_at[5] );


    /* floor 
     *
     */
    glBindTexture(GL_TEXTURE_2D, texture3 );
    glPushMatrix();
      glColor3f(1.0f, 1.0f, 1.0f);
      /*** the center arena floor ***/
      glBegin(GL_QUADS);  
        glVertex3f(-10.0f, -7.0f, -10.0f);
	glVertex3f( 10.0f, -7.0f, -10.0f);
	glVertex3f( 10.0f, -7.0f,  10.0f);
	glVertex3f(-10.0f, -7.0f,  10.0f);
      glEnd();
      /*** v0.1.8: a semblance of a back wall ***/
      glBegin(GL_QUADS);  
        glVertex3f(-10.0f, -3.0f, -10.0f);
	glVertex3f( 10.0f, -3.0f, -10.0f);
	glVertex3f( 10.0f, -6.6f, -10.0f);
	glVertex3f(-10.0f, -6.6f, -10.0f);
      glEnd();
      /*** v0.1.8: a semblance of a left wall ***/
      glBegin(GL_QUADS);
        glVertex3f(-10.0f, -3.0f, 10.0f);
	glVertex3f(-10.0f, -3.0f, -10.0f);
	glVertex3f(-10.0f, -6.6f, -10.0f);
	glVertex3f(-10.0f, -6.6f, 10.0f);
      glEnd();
      /*** v0.1.8: a semblance of a right wall ***/
      glBegin(GL_QUADS);
        glVertex3f(10.0f, -3.0f, -10.0f);
	glVertex3f(10.0f, -3.0f, 10.0f);
	glVertex3f(10.0f, -6.6f, 10.0f);
	glVertex3f(10.0f, -6.6f, -10.0f);
      glEnd();
      /*** v0.1.8: a semblance of a front wall ***/
      glBegin(GL_QUADS);  
        glVertex3f(-10.0f, -3.0f, 10.0f);
	glVertex3f( 10.0f, -3.0f, 10.0f);
	glVertex3f( 10.0f, -6.6f, 10.0f);
	glVertex3f(-10.0f, -6.6f, 10.0f);
      glEnd();
    glPopMatrix();








    /*** frame rate counter ***/
    if ( display_frame_rate == 1 ) {
      char pmes[256];
      sprintf( pmes, "%.3f fps", frames_per_second );
      draw_string( 10.0, 10.0, 1.0,
		   1.0f, 1.0f, 1.0f, 1.0f,
		   pmes, 10 );	
    }
    


    sprintf( screenmes, "%d", player1->victories );
    draw_string( 5.0, 40.0, 1.0,
		 1.0f, 1.0f, 1.0f, 1.0f,
		 screenmes, strlen(screenmes) );	
    
    sprintf( screenmes, "%d", player2->victories );
    draw_string( 610.0, 40.0, 1.0,
		 1.0f, 1.0f, 1.0f, 1.0f,
		 screenmes, strlen(screenmes) );	

    /* players 1 and 2
     *
     *
     */
    glBindTexture(GL_TEXTURE_2D, texture );


    /*** player 1 ***/
    glColor3f(1.0f, 0.0f, 0.0f);
    glPushMatrix();
      glTranslatef( player1->position[0],
		    player1->position[1],
		    player1->position[2] );
      display_entity( player1->opponent_rotation, player1->params );
    glPopMatrix();

    /*** player 2 ***/
    glColor3f(0.0f, 1.0f, 1.0f);
    glPushMatrix();
      glTranslatef( player2->position[0],
		    player2->position[1],
		    player2->position[2] );
      display_entity( player2->opponent_rotation, player2->params );
    glPopMatrix();


    glPopMatrix();



    /* v0.2.2: pause options menu
     *   note the resolution of the screen...
     *   also the bg color of the text.
     *
     *
     *
     */
    if ( running_state == RUNSTATE_PAUSE ) {
      char pmes[256];
      
#if 0
      /*** background for text ***/
      draw_box( 140.0, 100.0, 300.0, 300.0, 
		0.0, 0.0, 0.0, 0.0 );
      //1.0, 1.0, 1.0, 1.0 );

#else
      glColor3f(0.1f, 0.1f, 0.0f);
      glBindTexture(GL_TEXTURE_2D, texture3 );
      glBegin(GL_QUADS);  
      glVertex3f(-1.20f,   0.850f,   -1.0f);
      glVertex3f( 1.0f,   0.850f,   -1.0f);
      glVertex3f( 1.0f,  -0.80f,   -1.0f);
      glVertex3f(-1.20f,  -0.80f,   -1.0f);
      glEnd();
#endif
      
      
      sprintf( pmes, "http://www.ugrad.cs.jhu.edu/~susinno/dcfighting.html" );
      draw_string( 4.0, 70.0, 1.0,
		   1.0f, 1.0f, 1.0f, 1.0f,
		   pmes, strlen(pmes) );


      
      sprintf( pmes, "PAUSED" );
      draw_string( 100.0, 100.0, 1.0,
		   1.0f, 1.0f, 1.0f, 1.0f,
		   pmes, strlen(pmes) );


      
      sprintf( pmes, "up/down selects, A/X changes" );
      draw_string( 100.0, 120.0, 1.0,
		   1.0f, 1.0f, 1.0f, 1.0f,
		   pmes, strlen(pmes) );


      sprintf( pmes, "camera follow  %d", camera_follow );
      draw_string( 100.0, 170.0, 1.0,
		   1.0f, 1.0f,  1.0f, 1.0f, 
		   pmes, strlen(pmes) );

      sprintf( pmes, "block damage    %d", block_damage );
      draw_string( 100.0, 200.0, 1.0,
		   1.0f, 1.0f, 1.0f, 1.0f, 
		   pmes, strlen(pmes) );
      

      sprintf( pmes, "frame rate       %d", display_frame_rate );
      draw_string( 100.0, 230.0, 1.0,
		   1.0f, 1.0f,  1.0f, 1.0f, 
		   pmes, strlen(pmes) );
      
      sprintf( pmes, "use stamina       %d", use_stamina );
      draw_string( 100.0, 260.0, 1.0,
		   1.0f, 1.0f,  1.0f, 1.0f, 
		   pmes, strlen(pmes) );
    
      sprintf( pmes, "players            %d / %d", player_count, controller_count );
      draw_string( 100.0, 290.0, 1.0,
		   1.0f, 1.0f,  1.0f, 1.0f, 
		   pmes, strlen(pmes) );
 

      sprintf( pmes, "reset" );
      draw_string( 100.0, 320.0, 1.0,
		   1.0f, 1.0f,  1.0f, 1.0f, 
		   pmes, strlen(pmes) );
 
      /*** the hacky pointer ***/
      draw_string( 80.0, 170.0 + 30.0 * (float)menu_selection, 1.0,
		   1.0f, 1.0f,  1.0f, 1.0f, 
		   "*", 1 );
 

    }
    
    
    /* prompt for next game
     *
     *
     */
    if ( running_state == RUNSTATE_VICTORY ) {
      draw_string( 100.0, 170.0, 1.0,
		   1.0f, 1.0f,  1.0f, 1.0f, 
		   "press start to go again", strlen("press start to go again") );
    }


    



    /* Finish the frame */

    glKosFinishFrame();
    frame_count++;
  } /*** end main loop ***/
  
  return 0;
}


