/* dcf_player.c
 from dc fighting v0.2.0
  by
   jimbo
    Dan Potter and Jordan DeLong of Cryptic Allusion
     Andrew K.
      Marcus Comstedt



 contains:
   #define constants for fighting calculations in dcf_consequence.c
   dcfighting player struct and init routine
   



 */



/*** 0.1.2: state variables for combat! ***/
#define FIGHT_STAND   101
#define FIGHT_MOVE    95

/* above 100 state represents range, height and power:
 * 100s place is power
 * 10s place is height
 * 1s place is range
 *
 * combine attributes with sum
 *
 */

#define FIGHT_HIGH           10
#define FIGHT_MIDDLE         20
#define FIGHT_LOW            30

#define FIGHT_SHORT_RANGE   100
#define FIGHT_MEDIUM_RANGE  200
#define FIGHT_LONG_RANGE    300

#define FIGHT_BLOCK         100
#define FIGHT_PARRY         200
#define FIGHT_REPEL         300
#define FIGHT_COUNTER       400

#define FIGHT_THROW         900

#define FIGHT_DEFENSE      1001
#define FIGHT_ATTACK       2001

/*** below 100 state represents endurance... ***/
#define FIGHT_HIT            90
#define FIGHT_STUNNED        80
#define FIGHT_STAGGER        70
#define FIGHT_FALLING        30
#define FIGHT_KNEEL          20
#define FIGHT_DOWN           10


#define ENERGY_MAX 50
#define STAMINA_MAX 2000

/*** v0.1.9 partial param sets ***/
#define PARAMS_ALL       -1
#define PARAMS_SPATIAL    0
#define PARAMS_LOCATION   1
#define PARAMS_ROTATION   2
#define PARAMS_ARMS        3
#define PARAMS_LEGS          4 
#define PARAMS_HEAD          5
#define PARAMS_BACK         6


/*** player structure ***/
typedef struct {
  double future_lists[ NUMBER_PARAMETERS + OFFSET_HACK ][ FUTURE_SIZE ];
  int lists_in_future;
  int future_pointer;
  int reverse;
  double params[NUMBER_PARAMETERS];
  /*** v0.2.2 ***/
  double position[4];

  /*** these are for Dan Potter's button-hold logic ***/
  int trans;
  int trans2;
  int transa;
  int transb;
  int transx;
  int transy;
  int transyb;
  int transab;
  int transltrig;
  int transrtrig;
  int transup;
  int transdown;
  
  /*** 0.1.2: state variables for combat! ***/
  int fighting_state;
  int default_state;

  /*** these are to ease fighting logic ***/
  int power, height, range;
  int ducking;

  /*** v0.1.6: for input history ***/
  int controller_history[ HISTORY_SIZE ];
  int change_history[ HISTORY_SIZE ];
  int change_pointer;
  int history_pointer;


  /*** v0.1.7: lets add energy already ***/
  int energy;

  /* v0.2.4: adding limb stamina
   *
   *
   *
   */
  int stamina;
  int leg_stamina;


  /*** v0.2.3 ***/
  int victories;

  double opponent_rotation;

  void *opponent;
} fighter_t;





/*** v0.2.0 now here ***/
fighter_t *player1, *player2, *player3, *player4;



/*** set him up ***/
void init_fighter( fighter_t *p ) {
  int i;
  
  /*** v0.2.3 ***/
  p->victories = 0;
  reset_fighter( p );
}







/*** set him up ***/
void reset_fighter( fighter_t *p ) {
  int i;
  
  p->lists_in_future = 0;
  p->future_pointer = 0;
  p->reverse = 0;
  /*** set params in main ***/
  p->trans = 0;
  p->trans2 = 0;
  p->transa = 0;
  p->transb = 0;
  p->transx = 0;
  p->transy = 0;
  p->transyb = 0;
  p->transab = 0;
  p->transltrig = 0;
  p->transrtrig = 0;
  p->transdown = 0;
  p->fighting_state = FIGHT_STAND;
  p->default_state = FIGHT_STAND;
  p->change_pointer = 0;
  p->history_pointer = 0;
  /*** quick init of arrays ***/
  for ( i=0; i<HISTORY_SIZE; i++ ) {
    p->controller_history[i] = p->change_history[i] = 65535; /*** means no input ***/
  }


  
  // lose this and youre toast
  for ( i=0; i<NUMBER_PARAMETERS; i++ ) {
        p->params[i] = left_fighting_stance[i];
  }

  
  /*** position in space ***/
  for ( i=0; i<4; i++ ) {
    p->position[i] = 0.0;
  }


  for ( i=0; i<FUTURE_SIZE; i++ ) {
    p->future_lists[0][i] = 0.0;
  }

  p->energy = ENERGY_MAX;
  p->stamina = STAMINA_MAX;


  p->ducking = 0;
  p->opponent_rotation = 0.0;
}









/* line up a player to face his opponent
 *
 *
 */
void line_up_opponent( fighter_t *p ) {
  double xdot, zdot;
  double xaxis[] = {1.0,0.0,0.0};
  double zaxis[] = {0.0,0.0,1.0};
  double towards_vec[4];
  double up[4] = { 0.0, 1.0, 0.0, 1.0 };
  double cross_prod[4];
  double toward_norm;
  
  /*** v0.2.2; add vector away from opponent ***/
  towards_vec[0] = ( (fighter_t*)(p->opponent) )->position[0] - p->position[0];
  towards_vec[1] = ( (fighter_t*)(p->opponent) )->position[1] - p->position[1];
  towards_vec[2] = ( (fighter_t*)(p->opponent) )->position[2] - p->position[2];

  toward_norm = sqrt(
		     towards_vec[0] * towards_vec[0] +
		     towards_vec[1] * towards_vec[1] +
		     towards_vec[2] * towards_vec[2]
		     );
  
  towards_vec[0] /= toward_norm;
  towards_vec[1] /= toward_norm;
  towards_vec[2] /= toward_norm;
  
  cross_product( up, towards_vec, cross_prod );
  
  /* v0.1.9 : direction calculation
   *  dot p1p2 vector with x, then z
   *   take -atan2(xdot,zdot)
   *    good.
   */
  xdot = dot_product( towards_vec, xaxis );
  zdot = dot_product( towards_vec, zaxis );
  
  p->opponent_rotation = -atan2( xdot, zdot );
}
